from __future__ import print_function
import os
import sys
import logging
import argparse
from  bam2x import IO,TableIO,DBI
from  bam2x.Annotation import BED12
import logging

def help():
    return "Processing Query RNASeq Result, and count the number of compatible reads and overlap&not compatible reads. normalized by number of hits. Input is the output of query_RNASeq."
def set_parser(parser):
    #parser.add_argument("-m",type=str,choices=("seq","cDNA","cdna","cds","utr5","utr3"),dest="method")
    
    pass

def run(args):
    logging.basicConfig(level=logging.INFO)
    fin=IO.fopen(args.input,"r")
    out=IO.fopen(args.output,"w")
    print("GENE\tHIT\tOVERLAP",file=out);
    for qr,hits,overlap in iterate(fin):
        print("{qr_name}\t{hits_num:10.2f}\t{overlap_num:10.2f}".format(qr_name=qr.id,hits_num=count(hits),overlap_num=count(overlap)),file=out);
def count(bedlist):
    s=0.0
    for i in bedlist:
        x=float(i.itemRgb.split(",")[0])
        if x==0.0: x=1.0
        s+=1.0/x
    return s


def iterate(fin):
    buf=fin.next();
    x=buf.split("\t")[1:]
    qr=BED12._make(BED12._types(x))
    hits=[]
    overlap=[]
    i=0
    for x in TableIO.parse(fin):
        if x[0]=="QR":
            if i%100==0: 
                logging.info("processing "+str(i)+"  genes");
            i+=1
            yield qr,hits,overlap
            qr=BED12._make(BED12._types(x[1:]))
            hits=[]
            overlap=[]
        elif x[0]=="HT":
            hits.append(BED12._make(BED12._types(x[1:])))
        elif x[0]=="OP":
            overlap.append(BED12._make(BED12._types(x[1:])))
    yield qr,hits,overlap
        

if __name__=="__main__":
    from bam2x.IO import parser_factory
    p=parser_factory(description=help())
    set_parser(p)
    if len(sys.argv)==1:
        print(p.print_help())
        exit(0)
    run(p.parse_args())










from bam2x.Annotation import H_VCF,F_VCF
from bam2x.DBI.Templates import sqlite3_template_factory
schema,insert=sqlite3_template_factory(H_VCF,F_VCF)
import sqlite3
conn=sqlite3.connect("test.db")
cursor=conn.cursor()
cursor.execute(schema.substitute({"table_name":"test"}))
print insert.substitute({"table_name":"test"})
cursor.executemany(insert.substitute({"table_name":"test"}),[("chr1",200,"testvcf","A","C",40,"a","test")])

conn.commit()



from __future__ import print_function
import os
import sys
import logging
import argparse
from  bam2x import IO,TableIO,DBI
from bam2x.Tools import _translate_to_meta,merge_bed
def help():
    return "merged spliced bed ( Result from clean_reads )"
def set_parser(parser):
    #parser.add_argument("-m",type=str,choices=("seq","cDNA","cdna","cds","utr5","utr3"),dest="method")
    parser.add_argument("--gap",type=int,dest="gap",default=200)
    
def nh(bed): 
    '''
    Return 1.0/NH ( number of hits )  
    '''
    x=float(bed.itemRgb.split(",")[0])
    if x==0.0: x=1.0
    return 1.0/x
    
def run(args):
    def process():
        if len(buff)==1: return 0
        meta=buff[0]
        score=0.0
        score=nh(buff[0])
        for i in buff[1:]:
            meta=merge_bed(meta,i,"group_"+str(group_id))
            score+=nh(i)
        meta=meta._replace(score=score)

        print("META\t{meta}".format(meta=meta),file=out)
        for i in buff:
            print("READ\t{read}".format(read=_translate_to_meta(meta,i)),file=out)
        return 1
    GAP=args.gap
    fin=IO.fopen(args.input,"r")
    out=IO.fopen(args.output,"w")
    iterator=TableIO.parse(fin,"bed12")
    last=iterator.next()
    last_stop=last.stop
    group_id=0
    buff=[last]
    last_chr=last.chr
    for i in iterator:
        if i.chr!=last_chr or i.start-last_stop > GAP:
            group_id+=process()
            buff=[i]
            last_chr=i.chr
            last_stop=i.stop
        else:
            buff.append(i)
            if i.stop>last_stop:
                last_stop=i.stop
    process()
        

    
    
    
        

if __name__=="__main__":
    from bam2x.IO import parser_factory
    p=parser_factory(description=help())
    set_parser(p)
    if len(sys.argv)==1:
        print(p.print_help())
        exit(0)
    run(p.parse_args())










import string
from bam2x.Annotation import BED12,BED6,BED3
import itertools

hTypesToSqlite3 = {
    "int":"INTEGER",
    "long":"INTEGER",
    "float":"REAL",
    "double":"REAL",
    "text":"TEXT",
    "blob":"NONE",
    "boolean":"NUMERIC",
    "date":"NUMERIC"
}
def translate_to_sqlite3_format(f):
    if hTypesToSqlite3.has_key(f):
        return hTypesToSqlite3[f]
    else:
        return "TEXT"


def sqlite3_template_factory(ids,formats):
    schema="create table if not exists $table_name (\n"
    id_str=""
    v=""
    for i,f in itertools.izip(ids,formats):
        schema+="{id} {format},\n".format(id=i,format=translate_to_sqlite3_format(f.__name__))
        if i=="id" or i=="name":
            schema=schema.strip("\n")
            schema=schema.strip(",")
            schema+=" primary key,\n"
        id_str+=i+","
        v+="?,"
    id_str=id_str.strip(",")
    v=v.strip(",")
    schema=schema.strip("\n")
    schema=schema.strip(",")
    schema+=")"
    insert="insert into $table_name ({id_str}) values ({v})".format(id_str=id_str,v=v)
    return string.Template(schema),string.Template(insert)


bed12_schema_template= string.Template("""create table if not exists $table_name (
    id      integer primary key autoincrement not null,
    chr     varchar(20),
    start   integer,
    stop    integer,
    name    varchar(200),
    score   float,
    strand  character(1),
    cds_start   integer,
    cds_stop    integer,
    itemRgb     varchar(20),
    blockCount  integer,
    blockSizes  varchar,
    blockStarts varchar
)
""")

bed12_insert_template=string.Template("""insert into $table_name (chr,start,stop,name,score,strand,cds_start,cds_stop,itemRgb,blockCount,blockSizes,blockStarts)
                            values (?,?,?,?,?,?,?,?,?,?,?,?)
                            """)
 
bed12_select_template=string.Template("""select * from $table_name where name==$name""")


bed6_schema_template= string.Template("""create table if not exists $table_name (
    id      integer primary key autoincrement not null,
    chr     varchar(20),
    start   integer,
    stop    integer,
    name    varchar(200),
    score   float,
    strand  character(1)
)
""")

bed3_schema_template= string.Template("""create table if not exists $table_name (
    id      integer primary key autoincrement not null,
    chr     varchar(20),
    start   integer,
    stop    integer
)
""")

bed6_insert_template=string.Template("""insert into $table_name (chr,start,stop,name,score,strand)
                            values (?,?,?,?,?,?)
                            """)
 

bed3_insert_template=string.Template("""insert into $table_name (chr,start,stop)
                            values (?,?,?)
                            """)
 

schema_templates={
    "bed12":bed12_schema_template,
    "bed6":bed6_schema_template,
    "bed3":bed3_schema_template,
}
insert_templates={   
    "bed12":bed12_insert_template,
    "bed6":bed6_insert_template,
    "bed3":bed3_insert_template,
}

factories = {
"bed12":lambda c,r:BED12._make(BED12._types(r[1:])),
"bed6":lambda c,r:BED6._make(BED6._types(r[1:])),
"bed3":lambda c,r:BED3._make(BED3._types(r[1:])),
}


select_template=string.Template("""select * from $table_name where name=\"$name\"""")






#!/usr/bin/env python
from __future__ import print_function
import os
import sys
import logging
import argparse
from bam2x import TableIO,IO,DBI
from bam2x.Tools import compatible_with_transcript,_translate,get_flank_region,translate_coordinate,gini_coefficient
import math
import logging
from bam2x.Annotation import BED6,BED12

def help():
    return "aggregation plot figure near tss or tts"
def set_parser(parser):
    parser.add_argument("-b","--bed_db",type=str,dest="bed_db")
    parser.add_argument("-B","--bed_db_type",type=str,choices=["bed6","bed12"],default="bed6",dest="bed_db_type",help="bed db type ,default:%(default)s")
    parser.add_argument('-I','--input_format',dest="format",choices=("bed6","bed12"),default="bed12",type=str,help="input file format default=%(default)s")
    parser.add_argument("--up",type=int,dest="up",default=500,help="upstream bp")
    parser.add_argument("--down",type=int,dest="down",default=500,help="downstream bp")
    parser.add_argument("--tts",dest="tts",action="store_true",default=False,help="if this option is chosen , aggregate the tts near region")
    #parser.add_argument("--strand",type=str,dest="strand",choices=("read1","read2"),default="read1",help="paired end read strand, default: %(default)s")
  
def run(args):
    logging.basicConfig(level=logging.INFO)
    up=args.up
    down=args.down
    bp_num=up+down
    offset=-up
    bed_db=DBI.init(args.bed_db,"binindex",cls=args.bed_db_type)
    fin=IO.fopen(args.input,"r")
    out=IO.fopen(args.output,"w")
    bin_sum=[0 for i in xrange(bp_num)]
    bin_e=[0.0 for i in xrange(bp_num)]
    bin_dis=[[] for i in xrange(bp_num)]
    for i0,bed in enumerate(TableIO.parse(fin,args.format)):
        bed_bin=[0 for i in xrange(bp_num)]
        if args.tts:
            pos=bed.tts()
        else:
            pos=bed.tss()
        pos_flank=get_flank_region(pos,up,down)
        for read in bed_db.query(pos_flank):
            rstart,rend,rstrand=translate_coordinate(pos,read)
            #print(a,file=out)
                #print(e,file=out)
            start=rstart-offset
            end=rend-offset
            if start < 0: start=0
            if end > bp_num: end=bp_num
            for j in xrange(start,end):
                bed_bin[j]+=1
        for  i in xrange(bp_num):
            bin_sum[i]+=bed_bin[i]
            bin_dis[i].append(bed_bin[i])
    bed_num=i0+1
    for i in xrange(bp_num):
        bin_e[i]=gini_coefficient(bin_dis[i])
    if args.tts:
        print("pos_to_tts\taggregation_mean\tgini_coefficient",file=out)
    else:
        print("pos_to_tss\taggregation_mean\tgini_coefficient",file=out)
    for i in xrange(bp_num):
        print("{bin}\t{aggregation}\t{E}".format(bin=i+offset,aggregation=float(bin_sum[i])/bed_num,E=bin_e[i]),file=out)
    
    try:
        import matplotlib
        matplotlib.use('Agg')
        import matplotlib.pyplot as plt
        matplotlib.rcParams.update({'font.size':9})
        ax1=plt.subplot2grid((7,1),(6,0))
        plt.ylabel('gini coeffecient')
        plt.fill_between(range(-up,down),bin_e,color="r",alpha=0.2,y2=0)
        ax1.set_ylim(0,1)
        ax1.set_xlim(-up,down)
        ax1.axes.get_xaxis().set_visible(False)
        plt.axvline(x=0,linewidth=1, color='y')
        ax2=plt.subplot2grid((7,1),(0,0),rowspan=5)
        ax2.set_xlim(-up,down)
        plt.plot(range(-up,down),[float(i)/bed_num for i in bin_sum])
        plt.ylabel('mean coverage')
        if args.tts:
            plt.xlabel('pos to tts (bp)')
        else:
            plt.xlabel('pos to tss (bp)')
        plt.axvline(x=0,linewidth=1, color='y')
        plt.grid(True)
        plt.savefig(args.output+".png")
    except:
        print("wrong in print png",file=sys.stderr)
        pass








if __name__=="__main__":
    from bam2x.IO import parser_factory
    p=parser_factory(description=help())
    set_parser(p)
    if len(sys.argv)==1:
        print(p.print_help())
        exit(0)
    run(p.parse_args())




from __future__ import print_function
import os
import sys
import logging
import argparse
from  bam2x import IO,TableIO,DBI
from  bam2x.Annotation import BED12
import logging

def help():
    return "Processing Query RNASeq Result, and count the number of compatible reads and overlap&not compatible reads. normalized by number of hits. Input is the output of query_RNASeq."
def set_parser(parser):
    #parser.add_argument("-m",type=str,choices=("seq","cDNA","cdna","cds","utr5","utr3"),dest="method")
    
    pass

def run(args):
    logging.basicConfig(level=logging.INFO)
    fin=IO.fopen(args.input,"r")
    out=IO.fopen(args.output,"w")
    for qr,hits,overlap in iterate(fin):
        l=qr.cdna_length()
        pileup=[0.0 for i in xrange(l)]
        for i in hits:
            for j in xrange(i.start,i.stop):
                pileup[j]+=nh(i)
        print("QR\t{id}".format(id=qr.id),file=out)
        print("PILEUP\n{value}".format(value=rep(pileup)),file=out)
def rep(list):
    s=""
    last=list[0]
    step=0
    offset=0
    for i in list:
        if i!=last:
            s+="{offset},{value},{step}\n".format(offset=offset,value=last,step=step)
            last=i
            offset+=step
            step=0
        else:
            step+=1
    s+="{offset},{value},{step}".format(offset=offset,value=last,step=step)
    return s
            
def nh(bed): 
    '''
    Return 1.0/NH ( number of hits )  
    '''
    x=float(bed.itemRgb.split(",")[0])
    if x==0.0: x=1.0
    return 1.0/x

def iterate(fin):
    buf=fin.next();
    x=buf.split("\t")[1:]
    qr=BED12._make(BED12._types(x))
    print(qr)
    hits=[]
    overlap=[]
    i=0
    for x in TableIO.parse(fin):
        if x[0]=="QR":
            if i%100==0: 
                logging.info("processing "+str(i)+"  genes");
            i+=1
            yield qr,hits,overlap
            qr=BED12._make(BED12._types(x[1:]))
            hits=[]
            overlap=[]
        elif x[0]=="HT":
            hits.append(BED12._make(BED12._types(x[1:])))
        elif x[0]=="OP":
            overlap.append(BED12._make(BED12._types(x[1:])))
    yield qr,hits,overlap

        

if __name__=="__main__":
    from bam2x.IO import parser_factory
    p=parser_factory(description=help())
    set_parser(p)
    if len(sys.argv)==1:
        print(p.print_help())
        exit(0)
    run(p.parse_args())










#!/usr/bin/env python
from __future__ import print_function
# Programmer : zhuxp
# Date: 
# Last-modified: 05-13-2014, 13:10:38 EDT
VERSION="0.1.0"
import os,sys,argparse
from bam2x.Annotation import BED6 as Bed
from bam2x.Annotation import BED12 as Bed12
from bam2x import TableIO,Tools,IO
from bam2x import DBI
import time
import logging
import itertools
def help():
    return "cmp region with gene set(bed12 format) "
def set_parser(p):
    ''' This Function Parse the Argument '''
    p.add_argument('-g','--gene',dest="genetab",type=str,help="Gene File, BED12 Format ")
    p.add_argument('-s','--strand',dest="strand",type=str,choices=("read1","read2"),default="read2",help="strand read1 or read2 , default: %(default)s")
    p.add_argument('-u','--upstream',dest="upstream",type=int,default=3000,help="upstream bp number , default: 3000")
    p.add_argument('-d','--downstream',dest="downstream",type=int,default=3000,help="downstream bp number , default: 3000")
    
def run(local_args):
    '''
    IO TEMPLATE
    '''
    logging.basicConfig(level=logging.INFO);
    global args,out
    args=local_args
    out=IO.fopen(args.output,"w")
    fin=IO.fopen(args.input,"r")
    print("# This data was generated by program ",sys.argv[0]," (version: %s)"%VERSION,file=out)
    print("in bam2x ( https://github.com/nimezhu/bam2x )",file=out)
    print("# Date: ",time.asctime(),file=out)
    print("# The command line is :",file=out)
    print("#\t"," ".join(sys.argv),file=out)
    gene=DBI.init(args.genetab,"binindex",cls="bed12");
    upstream_list=[]
    downstream_list=[]
    exons_list=[]
    introns_list=[]
    utr3_list=[]
    utr5_list=[]
    for g in gene:
        upstream_list.append(g.upstream(args.upstream));
        downstream_list.append(g.downstream(args.downstream));
        for e in g.Exons():
            exons_list.append(e)
        for i in g.Introns():
            introns_list.append(i)
        if not (g.utr3() is None):
            utr3_list.append(g.utr3())
        if not (g.utr5() is None):
            utr5_list.append(g.utr5())
    upstream=DBI.init(upstream_list,"binindex",cls="bed6")
    downstream=DBI.init(downstream_list,"binindex",cls="bed6")
    exons=DBI.init(exons_list,"binindex",cls="bed6")
    introns=DBI.init(introns_list,"binindex",cls="bed6")
    utr3=DBI.init(utr3_list,"binindex",cls="bed6")
    utr5=DBI.init(utr5_list,"binindex",cls="bed6")
    dbs=[upstream,gene,utr5,exons,introns,utr3,downstream]
    db_names=["upstream","gene","utr5","exons","introns","utr3","downstream"]
    db_rep=["U","G","5","E","I","3","D"]
    h={}
    def report(f,total):
        print( "# "+" ".join(["{rep}:{name} ".format(name=i,rep=j) for i,j in itertools.izip(db_names,db_rep)]),file=out);
        print("# total reads: {total:.0f}".format(total=total),file=out);
        for i0 in sorted(h.keys()):
            print("{key}\t{value:.0f}\t{fraction:.2f}%".format(key=i0,value=h[i0],fraction=float(h[i0])*100/total),file=f);
    s=0.0;
    for i,x in enumerate(TableIO.parse(fin,"bam2bed12",strand=args.strand)):
        pattern=["_" for j in range(7)];
        if i%10000==0:
            logging.info("processed {number} reads".format(number=i));
            report(sys.stderr,s);
        for j,db in enumerate(dbs):
            if hit(db.query(x)):
                pattern[j]=db_rep[j]
        pattern_bytes="".join(pattern); 
        NH=int(x.itemRgb.split(",")[0])
        if NH==0: NH=1;
        nh=1.0/NH
        s+=nh
        if h.has_key(pattern_bytes):
            h[pattern_bytes]+=nh
        else:
            h[pattern_bytes]=nh
    report(out,s)
    



def hit(x):
    for i in x:
        return True
    return False



if __name__=="__main__":
    from bam2x.IO import parser_factory
    p=parser_factory(description=help())
    set_parser(p)
    if len(sys.argv)==1:
        print(p.print_help())
        exit(0)
    run(p.parse_args())

    








from __future__ import print_function
import os
import sys
import logging
import argparse
from  bam2x import IO,TableIO,DBI
from  bam2x.Annotation import BED6
from bam2x.Tools import _translate_to_meta,merge_bed,gini_coefficient
from math import log
from bam2x.gmm import model_str,fit_two_peaks_EM,bayes_p2
from numpy import array
import time
VERSION="0.2"
def help():
    return "merged tss or tts (Result from bam2ends.py)"

def set_parser(parser):
    #parser.add_argument("-m",type=str,choices=("seq","cDNA","cdna","cds","utr5","utr3"),dest="method")
    parser.add_argument("--gap",type=int,dest="gap",default=10,help="min gap default:%(default)s")
    parser.add_argument("--min",type=int,dest="min_reads_number",default=0,help="reads number threshold, default:%(default)s")
    parser.add_argument("--gene",type=str,dest="gene",default=0,help="known gene bed12 file")
    parser.add_argument("--tts",action="store_true",dest="tts",help="tts default: tss")
    parser.add_argument("--prefix",dest="prefix",help="prefix for bed name",default="meta")
    

def run(args):
    logging.basicConfig(level=logging.INFO)
    def process():
        if len(buff)==1: return 0
        max_score=0.0
        total_score=0.0
        e=[]
        for i in buff:
            total_score+=i.score
            e.append(i.score)
        e=[i/total_score for i in e]
        gini=gini_coefficient(e)
        if total_score < args.min_reads_number:
            return 0
        record={}
        meta=BED6(buff[0].chr,buff[0].start,buff[-1].stop,args.prefix+"."+str(group_id),total_score,buff[0].strand)
        peak=max(buff,key=lambda x:x.score)
        record["peak"]=peak._replace(score=peak.score/total_score)
        record["meta"]=meta._replace(strand=peak.strand)
        record["gini"]=gini
        records.append(record)
        return 1
    
    
    def simple_output():
        print("# formats: bayes_prob_model2, gini, [ region bed, score is total reads], [peak bed , score is proportion ]",file=out)
        for i,x in enumerate(records):
            print("{p2}\t{gini}\t{meta}\t".format(p2=p2[i],meta=x["meta"],gini=x["gini"]),end="",file=out)
            print(x["peak"],file=out)
    def bed12_output():
        print("# formats: bed12 , [R,G,B] are corresponding to [ TTS_GINI_PVALUE*200, TSS_GINI_PALUE*200, PROPORTION_OF_PEAK*200 ]",file=out)
        for i,x in enumerate(records):
            if args.tts:
                g=0
                r=int(p2[i]*200)
            else:
                g=int(p2[i]*200)
                r=0
            b=int(x["gini"]*200)
            if p2[i]>0.5:
                meta=x["meta"]._replace(id=x["meta"].id+".end")
            else:
                meta=x["meta"]
            rgb="{r},{g},{b}".format(r=r,g=g,b=b)
            print("{bed6}\t{thickStart}\t{thickEnd}\t{itemRgb}\t{blockCount}\t{blockSizes}\t{blockStarts}".format(bed6=meta,thickStart=x["peak"].start,thickEnd=x["peak"].end,itemRgb=rgb,blockSizes=x["meta"].stop-x["meta"].start,blockCount=1,blockStarts=0),file=out)

    
    records=[]
    GAP=args.gap
    fin=IO.fopen(args.input,"r")
    out=IO.fopen(args.output,"w")
    iterator=TableIO.parse(fin,"bed6")
    last=iterator.next()
    last_stop=last.stop
    group_id=0
    buff=[last]
    last_chr=last.chr
    for x,i in enumerate(iterator):
        if x%10000==0: logging.info("processing {x} reads".format(x=x));
        if i.chr!=last_chr or i.start-last_stop > GAP:
            group_id+=process()
            buff=[i]
            last_chr=i.chr
            last_stop=i.stop
        else:
            buff.append(i)
            if i.stop>last_stop:
                last_stop=i.stop

    process()
    gini=array([i["gini"] for i in records])
    model=fit_two_peaks_EM(gini)
    p2=bayes_p2(gini,model)
    print("# Date: ",time.asctime(),file=out)
    print("# Program Version ",VERSION,file=out)
    print("# The command line is :",file=out)
    print("#\t"," ".join(sys.argv),file=out)
    print("# learning model:",file=out)
    print("#",model_str(model),file=out)
    #simple_output()
    bed12_output()
        

    
    
    
        

if __name__=="__main__":
    from bam2x.IO import parser_factory
    p=parser_factory(description=help())
    set_parser(p)
    if len(sys.argv)==1:
        print(p.print_help())
        exit(0)
    run(p.parse_args())










#TODO 
from __future__ import print_function
import os
import sys
import logging
import argparse
from  bam2x import IO,TableIO,DBI
from bam2x.Struct import binindex
from bam2x.Tools import compatible_with_transcript
def help():
    return "compare RNA Seq bam with Gene annotations, remove the low score (<20 ) and reads from gene"
def set_parser(parser):
    #parser.add_argument("-m",type=str,choices=("seq","cDNA","cdna","cds","utr5","utr3"),dest="method")
    parser.add_argument("-a","--bed",type=str,dest="bed",help="gene annotations bed12 file")
    parser.add_argument("-s","--strand",type=str,dest="strand",choices=("read1","read2"),default="read2",help="paired end RNA seq strand, default:%(default)s")
    
    
def run(args):
    SCORE_THRESHOLD=20
    logging.basicConfig(level=logging.INFO)
    out=IO.fopen(args.output,"w")
    logging.info("reading annotation into data structure")
    binindex_db=binindex(TableIO.parse(args.bed,"bed12"))
    logging.info("done.")
    mapped_reads_count=0
    compatible_reads_count=0
    overlap_not_compatible_reads_count=0
    no_overlap_reads_count=0
    spliced_reads_count=0
    for count,i in enumerate(TableIO.parse(args.input,"bam2bed12",strand=args.strand)):
        overlap_gene=False
        compatible_gene=False
        mapped_reads_count+=1
        if count%1000==0:
            print("M: {mapped}    \t  C: {compatible}  \t  O: {overlap}   \tN: {no_overlap}   \tS: {spliced}                               ".format(mapped=mapped_reads_count,compatible=compatible_reads_count,overlap=overlap_not_compatible_reads_count,no_overlap=no_overlap_reads_count,spliced=spliced_reads_count),file=sys.stderr,end="\r")
        if i.score<SCORE_THRESHOLD:
            continue
        if i.blockSize>1:
            spliced_reads_count+=1
        for j in binindex_db.query(i):
            overlap_gene=True
            if compatible_with_transcript(i,j):
                compatible_gene=True
                break
        if compatible_gene:
            compatible_reads_count+=1
            continue
        elif overlap_gene:
            overlap_not_compatible_reads_count+=1
            print(i,file=out)
        else:
            no_overlap_reads_count+=1
            print(i,file=out)

        print("M: {mapped}    \t  C: {compatible}  \t  O: {overlap}   \tN: {no_overlap}   \tS: {spliced}                               ".format(mapped=mapped_reads_count,compatible=compatible_reads_count,overlap=overlap_not_compatible_reads_count,no_overlap=no_overlap_reads_count,spliced=spliced_reads_count))








if __name__=="__main__":
    from bam2x.IO import parser_factory
    p=parser_factory(description=help())
    set_parser(p)
    if len(sys.argv)==1:
        print(p.print_help())
        exit(0)
    run(p.parse_args())





# -*- coding: utf-8 -*-
from numpy import random
from numpy.random import normal
from numpy import concatenate,array,zeros,r_,ones,log,sqrt,pi,exp,abs
import sys
import math
import logging
_norm_pdf_C = math.sqrt(2*pi)
def normpdf(x, mu=0.0, sigma=1.0):
    u = (x-mu)/abs(sigma)
    y = exp(-u*u/2.0)/(_norm_pdf_C*abs(sigma))
    return y

def model_str(p):
    return "mu_1:{u1:.3f},sig_1:{s1:.3f},pi_1:{p1:.3f}\tmu_2:{u2:.3f},sig_2:{s2:.3f},pi_2:{p2:.3f}".format(u1=p[0],s1=p[1],p1=p[4],u2=p[2],s2=p[3],p2=1-p[4])
def pdf_model(x, p):
    mu1, sig1, mu2, sig2, pi_1 = p
    return pi_1*normpdf(x, mu1, sig1) + (1.0-pi_1)*normpdf(x, mu2, sig2)
def bayes_p1(x,p): # bayes prob on modle 1 ,  
    mu1, sig1, mu2, sig2, pi_1 = p
    p1=pi_1*normpdf(x,mu1,sig1)
    p2=(1.0-pi_1)*normpdf(x,mu2,sig2)
    return p1/(p1+p2)
def bayes_p2(x,p):
    return 1.0-bayes_p1(x,p)
def log_likelihood_two_1d_gauss(p, sample):
    return -log(pdf_model(sample, p)).sum()
def sim_two_gauss_mix(p, N=1000): 
    s1 =  normal(p[0], p[1], size=N*p[4])
    s2 = normal(p[2], p[3], size=N*(1-p[4]))
    return concatenate([s1,s2])
def fit_two_peaks_EM(sample, sigma=None, weights=False, p0=array([0.2,0.2,0.7,0.2,0.5]), 
        max_iter=300, tollerance=1e-5):
    if not weights: w = ones(sample.size)
    else: w = 1./(sigma**2)
    w *= 1.*w.size/w.sum() # renormalization so they sum to N
    # Initial guess of parameters and initializations
    mu = array([p0[0], p0[2]])
    sig = array([p0[1], p0[3]])
    pi_ = array([p0[4], 1-p0[4]])
    gamma, N_ = zeros((2, sample.size)), zeros(2)
    p_new = array(p0)
    N = sample.size
    # EM loop
    counter = 0
    converged, stop_iteration = False, False
    while not stop_iteration:
        p_old = p_new
        # Compute the responsibility func. and new parameters
        for k in [0,1]:
            gamma[k,:] = w*pi_[k]*normpdf(sample, mu[k], sig[k])/pdf_model(sample, p_new) # SCHEME1
            #gamma[k,:] = pi_[k]*normpdf(sample, mu[k], sig[k])/pdf_model(sample, p_new) # SCHEME2
            N_[k] = gamma[k,:].sum()
            mu[k] = sum(gamma[k]*sample)/N_[k] # SCHEME1
            #mu[k] = sum(w*gamma[k]*sample)/sum(w*gamma[k]) # SCHEME2
            sig[k] = sqrt( sum(gamma[k]*(sample-mu[k])**2)/N_[k] )
            pi_[k] = 1.*N_[k]/N
        p_new = array([mu[0], sig[0], mu[1], sig[1], pi_[0]])
        assert abs(N_.sum() - N)/float(N) < 1e-6 
        assert abs(pi_.sum() - 1) < 1e-6
        # Convergence check
        counter += 1
        max_variation = max((p_new-p_old)/p_old)
        converged = True if max_variation < tollerance else False
        stop_iteration = converged or (counter >= max_iter)
    #print "Iterations:", counter
    if not converged: logging.warning("WARNING: Not converged")
    return p_new



if __name__=="__main__":
    import logging
    logging.basicConfig(level=logging.WARNING)
    p=[0.2,0.3,0.7,0.2,0.3]
    s = sim_two_gauss_mix(N=1000, p=p)
    print "data:",model_str(p)
    model=fit_two_peaks_EM(s)
    print "em  :",model_str(model)
    r=normal(p[0],p[1],size=10)
    print r,"\n",pdf_model(r,model)
    print bayes_p1(r,model)
    print bayes_p2(r,model)

